from django.test import TestCase
from django.contrib.auth.models import User
from django.conf import settings
from app.models import Item
from activation.models import ActivationKey


class ActivationKeyTestCase(TestCase):

    def setUp(self):
        # Create base user for tests.
        self.user = User.objects.create_user(
            username='pparamo', email='pparamo@rulfo.com', password='rulfo')

    def test_create_activation_key(self):
        " Test the creation of the key."

        key = ActivationKey.objects.generate(self.user)

        self.assertEqual(key.user, self.user)

        user = User.objects.get(id=self.user.id)
        self.assertFalse(user.is_active, "User must be inactive")

    def test_fire_created_signal(self):
        " Fire the signal of created activation key."
        self.assertEqual(Item.objects.count(), 0)
        ActivationKey.objects.generate(self.user)
        self.assertEqual(Item.objects.count(), 1)

    def test_invalid_activation_key(self):
        " Get an invalidated activation key."

        settings.ACTIVATION_VALID_TIME = 0
        key = ActivationKey.objects.generate(self.user)
        self.assertRaises(
            ActivationKey.DoesNotExist, ActivationKey.objects.get_valid,
            key.key)
        settings.ACTIVATION_VALID_TIME = 7

    def test_activate_user(self):
        " Activate an user."

        key = ActivationKey.objects.generate(self.user)
        key.activate()

        user = User.objects.get(id=self.user.id)
        self.assertTrue(user.is_active)
        self.assertEqual(ActivationKey.objects.filter(id=key.id).count(), 1)

    def test_activate_user_with_autodelete(self):
        " Activate an user with autodelete"

        settings.ACTIVATION_AUTODELETE = True
        key = ActivationKey.objects.generate(self.user)
        key.activate()

        user = User.objects.get(id=self.user.id)
        self.assertTrue(user.is_active)
        self.assertEqual(ActivationKey.objects.filter(id=key.id).count(), 0)
        settings.ACTIVATION_AUTODELETE = False



from setuptools import setup, find_packages
from os.path import join, abspath, dirname

fd = open(join(abspath(dirname(__file__)), 'README.rst'))
long_description = fd.read()
fd.close()

setup(
    name='django-activation',
    description='Simple activation of users.',
    long_description=long_description,
    version='0.1.0b',
    author='Jorge Eduardo Cardona Gaviria',
    author_email='jorgeecardona@gmail.com',
    packages=find_packages(),
    install_requires=[
        'django',
        ])



from django.db import models
from activation.models import ActivationKey
from activation.signals import activation_created
from django.dispatch import receiver


class Item(models.Model):
    #: content
    content = models.TextField()


@receiver(activation_created, sender=ActivationKey)
def create_item(sender, activation, **kwargs):
    Item.objects.create(content=activation.key)



from django.db import models
from django.conf import settings
from string import letters, digits
from random import choice
from datetime import datetime, timedelta
from activation import signals

KEY_LENGTH = getattr(settings, 'ACTIVATION_KEY_LENGTH', 80)
VALID_TIME = getattr(settings, 'ACTIVATION_VALID_TIME', 7)


class ActivationKeyManager(models.Manager):

    def _generate_key(self):
        return ''.join(choice(letters + digits) for i in range(KEY_LENGTH))

    def generate(self, user):
        " Generate an activation key to an user."

        key = self._generate_key()
        while self.filter(key=key).count() > 0:
            key = self._generate_key()

        # Create activation key.
        activation_key = self.create(user=user, key=key)

        # Deactivate user.
        user.is_active = False
        user.save()

        # Fire signal.
        signals.activation_created.send(
            sender=ActivationKey, activation=activation_key)
        return activation_key

    def get_valid(self, key):
        " Get a valid key."
        return self.get(
            key=key, creation_date__lt=datetime.now() - timedelta(VALID_TIME))


class ActivationKey(models.Model):

    #: User to activate.
    user = models.ForeignKey('auth.User', related_name='activation_keys')

    #: Key.
    key = models.CharField(max_length=KEY_LENGTH)

    #: Creation date.
    creation_date = models.DateTimeField(auto_now_add=True)

    #: Objects Manager.
    objects = ActivationKeyManager()

    def activate(self):
        " Activate the user."

        # Activate the user.
        self.user.is_active = True
        self.user.save()

        # Delete this activation key.
        AUTODELETE = getattr(settings, 'ACTIVATION_AUTODELETE', False)
        if AUTODELETE:
            self.delete()
